<?php 
/* * ************************************************************************
 *  
 *  Plugin Name: SimpleEvents
 *  Description: Simple events manager
 *  Version: 1.0.0
 *  Author: Zofia Federowicz-Bolczak
 *  Author URI: http://zofiafederowicz.com/
 *  
 * * ************************************************************************ */

# get correct id for plugin
$thisfile = basename(__FILE__, ".php");

# include plugin's config file if exists
$simpleevents_config = GSDATAOTHERPATH.'simpleevents/config.php';
if (file_exists($simpleevents_config)) include_once $simpleevents_config;

# add translation
i18n_merge('simpleevents') || i18n_merge('simpleevents', 'en_US');

# register plugin
register_plugin(
  $thisfile,
  'SimpleEvents',
  '1.0.0',
  'Zofia Federowicz-Bolczak',
  'http://zofiafederowicz.com/SimpleEvents-plugin/',
  i18n_r('simpleevents/PLUGIN_DESCRIPTION'),
  'pages',
  'simpleevents'
);

# add a link in the admin tab 'pages' (Back-End)
add_action('pages-sidebar', 'createSideMenu',
      array($thisfile, i18n_r('simpleevents/SIDEBAR')));

# add JavaScript to the pages site for highlighting events' list (Back-End)
add_action('footer', 'show_events');

# add styles for event's manager and event's fields' editor (Back-End)
add_action('header', 'add_styles');

# add event's fields to event's edit page (Back-End)
add_action('edit-extras', 'add_events_fields');

# save event's fields values (Back-End)
add_action('changedata-save', 'save_events_fields');

# add event's fields values to every event's page (Front-End)
add_action('content-top', 'add_events_fields_values');

# display events' list on events' parent page (Front-End)
add_filter('content','display_events_list');

# functions

function simpleevents() {
  global $live_plugins, $pagesArray;
  $i18n_plugin = 'i18n_navigation.php';
  if (isset($live_plugins[$i18n_plugin]) && $live_plugins[$i18n_plugin]) {
    # save event's parent page in SimpleEvents config file
    if (!defined('EVENTSPARENT') && isset($_POST['eventsparent'])
      && array_key_exists($_POST['eventsparent'], return_i18n_pages())) {
      $eventsparent = check_input($_POST['eventsparent']);
      $config_path = GSDATAOTHERPATH.'simpleevents/';
      $config_name = 'config.php';
      if (!file_exists($config_path.$config_name)) {
        if (!file_exists($config_path)) mkdir($config_path, 0775);
        $config_file = fopen($config_path.$config_name, 'w');
        $txt = "<?php\n"
          ."# slug of event's parent page\n"
          ."define('EVENTSPARENT', '".$eventsparent."');\n"
          ."?>";
        fwrite($config_file, $txt);
        fclose($config_file);
        chmod($config_path.$config_name, 0775);
      }
      # include plugin's config file
      include_once GSDATAOTHERPATH.'simpleevents/config.php';
    }
    if (defined('EVENTSPARENT')) {
      echo '<h3>'.i18n_r('simpleevents/TITLE').'</h3>'
        .'<h5><a href="edit.php?parent='.EVENTSPARENT.'">'
        .i18n_r('simpleevents/ADD').'</a></h5>'
        .'<h5><a href="load.php?id=i18n_base&simpleevents=list">'
        .i18n_r('simpleevents/LIST').'</a></h5>'
        .'<div id="SimpleEvents-main">'
        .'<h2>'.i18n_r('simpleevents/USERGUIDE').'</h2>'
        .i18n_r('simpleevents/USERGUIDE_DESCRIPTION')
        .'</div>';
    } else {
      # choose page to become event's parent
      echo '<h3>'.i18n_r('simpleevents/CONFIGURATION').'</h3>'
        .'<p>'.i18n_r('simpleevents/CHOOSE_PARENT').':</p>'
        .'<form action="load.php?id=simpleevents" method="post" '
        .'accept-charset="utf-8"><p>'
        .'<select name="eventsparent" class="text autowidth">';
      foreach (return_i18n_pages() as $slug=>$page) {
        if ($slug != '') {
          echo '<option value="'.$slug.'">'.$page['title'].'</option>';
        }
      }
      echo '</select>'
        .'&nbsp;&nbsp;&nbsp;<input type="submit" class="submit"'
        .'value="'.i18n_r('simpleevents/CHOOSE_PARENT_BUTTON').'" />'
        .'</p></form>';
    }
  } else {
    echo '<p>'.i18n_r('simpleevents/NO_I18N_PLUGIN').'</p>';
  }
}

function show_events() {
  if (defined('EVENTSPARENT')
    && isset($_GET['simpleevents']) && $_GET['simpleevents'] == 'list') {
    echo '<style type="text/css">'
      .'#tr-'.EVENTSPARENT.'{background-color:rgb(128,203,255);}'
      .'</style>';
    echo '<script type="text/javascript">'
      .'function add_border(element, color = "light") {'
      .'if (!element.hasClass("top")) {'
	  .'if (color == "light") {'
	  .'bg_color = "rgb(205,235,255)";'
	  .'next_color = "dark";'
	  .'} else {'
	  .'bg_color = "rgb(179,224,255)";'
	  .'next_color = "light";'
	  .'}'
	  .'element.css("background-color", bg_color);'
      .'add_border(element.next("tr"), next_color);'
      .'} else return false;'
      .'}'
      .'$(document).ready(function() {'
      .'$("#tr-'.EVENTSPARENT.' .dirtoggle").trigger("click");'
      .'var event_next = $("#tr-'.EVENTSPARENT.'").next("tr");'
      .'add_border(event_next);'
      .'});'
      .'</script>';
  }
}

function add_styles() {
  if (isset($_GET['id']) && $_GET['id'] == 'simpleevents') {
    echo '<style type="text/css">'
      .'#SimpleEvents-main{margin-top:2em;}'
      .'#SimpleEvents-main h4 {padding-bottom:1em;'
      .'font-size:larger;font-weight:bold;}'
      .'</style>';
  } elseif (get_event_details()['display'] == true) {
    echo '<style type="text/css">'
      .'#SimpleEvents-fields{'
      .'margin-top:1.5em;padding:1.5em 0;border-top:1px dotted #000;}'
      .'#SimpleEvents-fields label,#SimpleEvents-fields input{'
      .'display:inline;font-size:140%;}'
      .'#SimpleEvents-fields input.date,#SimpleEvents-fields input.time{width:9em;}'
      .'#SimpleEvents-fields input.loc{width:20em;}'
      .'</style>';
  }
}

function add_events_fields() {
  $event = get_event_details();
  if ($event['display'] == true) {
    echo '<div id="SimpleEvents-fields" class="wideopt">';
    if ($event['is_lang_def']) {
      echo '<p><label>'.i18n_r('simpleevents/DATE').':&nbsp;</label>'
        .'<input type="date" name="post-event-date" '
        .'value="'.$event['date'].'" class="text date" />'
        .'<label>&nbsp;&nbsp;'.i18n_r('simpleevents/DATETO').':&nbsp;</label>'
        .'<input type="date" name="post-event-date-to" '
        .'value="'.$event['date_to'].'" class="text date" /></p>'
        .'<p><label>'.i18n_r('simpleevents/TIME').':&nbsp;</label>'
        .'<input type="time" name="post-event-time" '
        .'value="'.$event['time'].'" class="text time" /></p>';
    }
    echo '<p><label>'.i18n_r('simpleevents/LOCATION').':&nbsp;</label>'
      .'<input type="text" name="post-event-loc" '
      .'value="'.$event['loc'].'" class="text loc" /></p>';
    if ($event['is_lang_def'])
      echo '<p>'.i18n_r('simpleevents/DATETIMEFIELDS_DESCRIPTION').'</p>';
    echo '</div>';
  }
}

function save_events_fields() {
  global $xml;
  $post_date = '';
  $post_date_to = '';
  $post_time = '';
  $post_loc = '';
  if (isset($_POST['post-event-date']))
    $post_date = check_input($_POST['post-event-date']);
  if (isset($_POST['post-event-date-to']))
    $post_date_to = check_input($_POST['post-event-date-to']);
  if (isset($_POST['post-event-time']))
    $post_time = check_input($_POST['post-event-time']);
  if (isset($_POST['post-event-loc']))
    $post_loc = check_input($_POST['post-event-loc']);
  require_once(GSPLUGINPATH.'simpleevents/SimpleEvent.class.php');
  $event = new SimpleEvent();
  $xml = $event->saveEventsFields($xml, $post_date, $post_date_to,
                                  $post_time, $post_loc);
}

function add_events_fields_values() {
  global $language;
  if (defined('EVENTSPARENT') && get_parent(false) == EVENTSPARENT) {
    require_once(GSPLUGINPATH.'simpleevents/SimpleEvent.class.php');
    if ($language != I18N_DEFAULT_LANGUAGE) $lang = '_'.$language;
    else $lang = '';
    $event = new SimpleEvent(get_page_slug(false).$lang);
    $event_dates = $event->getBothDates();
    # time formats for different languages
    $time_format_arr = array(
      'pl' => 'H:i',
      'en' => 'h:i a',
      'fr' => 'H \h i',
      'de' => 'H:i');
    $format_exists = array_key_exists($language, $time_format_arr);
    if ($format_exists) $time_format = $time_format_arr[$language];
    else $time_format = 'H:i';
    $event_time = $event->getTime($time_format);
    $event_loc = $event->location;
    if ($event_dates != '') echo '<h2 class="date">'.$event_dates.'</h2>';
    if ($event_time != '') echo '<h2 class="time">'.$event_time.'</h2>';
    if ($event_loc != '') echo '<h2 class="location">'.$event_loc.'</h2>';
  }
}

function display_events_list($content) {
  if (defined('EVENTSPARENT') && return_page_slug() == EVENTSPARENT) {
    $events = return_i18n_menu_data(EVENTSPARENT, 1, 1, I18N_SHOW_PAGES);
    if (!empty($events)) {
      $events_list = array();
      $past_events_list = array();
      $i = 0;
      foreach ($events as $event) {
        $url = get_page_url(true).$event['url'];
        $title = $event['title'];
        require_once(GSPLUGINPATH.'simpleevents/SimpleEvent.class.php');
        $event_obj = new SimpleEvent($event['url']);
        $date = $event_obj->date;
		    $date_to = $event_obj->dateTo;
        $dates = $event_obj->getBothDates('%e %b %Y');
        $today = new DateTime();
        $today->setTime(0,0);
		    if ($date == '' || $date >= $today 
		        || ($date_to != '' && $date_to >= $today)) {
          $events_list[$event_obj->getDate().$i] = array(
            'url' => $url, 'title' => $title, 'dates' => $dates);
        } else {
          $past_events_list[$event_obj->getDate().$i] = array(
            'url' => $url, 'title' => $title, 'dates' => $dates);
        }
        $i++;
      }
      krsort($events_list);
      krsort($past_events_list);
      if (!empty($events_list)) events_list_echo($events_list);
      else echo $content;
      events_list_echo($past_events_list, true);
    } else {
      return $content;
    }
  } else {
    return $content;
  }
}

# helper functions

function check_input($data) {
  $data = trim($data);
  $data = stripslashes($data);
  $data = htmlspecialchars($data);
  return $data;
}

function get_event_details() {
  $display = false;
  $is_lang_def = true;
  $date = '';
  $date_to = '';
  $time = '';
  $loc = '';
  $php_self_arr = preg_split('/\\\|\//', $_SERVER['PHP_SELF']);
  if (defined('EVENTSPARENT') && end($php_self_arr) == 'edit.php') {
    if (isset($_GET['id']) || isset($_GET['newid'])) {
      require_once(GSPLUGINPATH.'simpleevents/SimpleEvent.class.php');
      if (isset($_GET['id'])) $url = $_GET['id'];
      if (isset($_GET['newid'])) $url = $_GET['newid'];
      $event = new SimpleEvent($url);
      $is_lang_def = $event->isLangDef;
      if ($event->parent == EVENTSPARENT) {
        $display = true;
        $date = $event->getDate();
        $date_to = $event->getDateTo();
        $time = $event->getTime();
        $loc = $event->location;
      }
    } elseif (isset($_GET['parent']) && $_GET['parent'] == EVENTSPARENT) {
      $display = true;
    }
  }
  return array(
    'display' => $display,
    'is_lang_def' => $is_lang_def,
    'date' => $date,
    'date_to' => $date_to,
    'time' => $time,
    'loc' => $loc);
}

function events_list_echo($list, $is_past = false) {
  $past = ($is_past) ? ' SimpleEvents-past' : '';
  echo '<ul class="SimpleEvents'.$past.'">';
  foreach ($list as $event) {
    $dash = ($event['dates'] != '') ? ' - ': '';
    echo '<li><span>'.$event['dates'].'</span>'
      .'<a href="'.$event['url'].'">'.$event['title'].'</a></li>';
  }
  echo '</ul>';
}
?>
