<?php

class SimpleEvent {

  public $parent;
  public $date;
  public $dateTo;
  public $time;
  public $location;
  public $isLangDef = true;

  public function __construct($url = false) {
    if ($url != false) {
      # get event's details from an event's XML file
      if (file_exists(GSDATAPAGESPATH.$url.'.xml')) {
        $eventXML = getXML(GSDATAPAGESPATH.$url.'.xml');
        $this->location = $eventXML->eventLocation;
        $this->parent = $eventXML->parent;
      }
      if (strpos($url, '_')) {
        $this->isLangDef = false;
        $slug = explode('_', $url)[0];
        if (file_exists(GSDATAPAGESPATH.$slug.'.xml')) {
          $eventXML = getXML(GSDATAPAGESPATH.$slug.'.xml');
          $this->parent = $eventXML->parent;
        }
      }
      if (file_exists(GSDATAPAGESPATH.$url.'.xml')) {
        $this->date = $this->getDateTimeObj($eventXML->eventDate);
        $this->dateTo = $this->getDateTimeObj($eventXML->eventDateTo);
        $this->time = $this->getDateTimeObj($eventXML->eventTime);
      }
    }
  }

  # helper function creating a DateTime object from a string
  private function getDateTimeObj($data) {
    if ($data != '') {
      try {
        $data = new DateTime($data);
      } catch (Exception $e) {}
    }
    return $data;
  }

  # saving event's details to an event's XML file
  public function saveEventsFields($xml, $date, $date_to, $time, $location) {
    if (!empty($date)) {
      $this->date = $this->getDateTimeObj($date);
      $this->addXmlChild($xml, 'eventDate', $this->getDate(), $date);
    }
    if (!empty($date_to)) {
      $this->dateTo = $this->getDateTimeObj($date_to);
      $this->addXmlChild($xml, 'eventDateTo', $this->getDateTo(), $date_to);
    }
    if (!empty($time)) {
      $this->time = $this->getDateTimeObj($time);
      $this->addXmlChild($xml, 'eventTime', $this->getTime(), $time);
    }
    if (!empty($location)) {
      $this->addXmlChild($xml, 'eventLocation', stripslashes($location));
    }
    return $xml;
  }

  # helper function adding a child element and its contents to an XML node
  private function addXmlChild($xml, $name, $value, $value_post = false) {
    if ($value == $value_post || !$value_post) {
      $event = $xml->addChild($name);
      $event->addCData($value);
    }
  }

  # get an event's date (date from)
  public function getDate($format = 'Y-m-d') {
    return $this->formatDate($this->date, $format);
  }

  # get an event's date (date to)
  public function getDateTo($format = 'Y-m-d') {
    return $this->formatDate($this->dateTo, $format);
  }

  # get event's both dates (from-to)
  public function getBothDates($format = '%A, %e %B %Y') {
    # Polish months' names
    $pl_mo = array(
      'm'=>array('styczeń','luty','marzec','kwiecień',
                 'maj','czerwiec','lipiec', 'sierpień',
                 'wrzesień','październik','listopad','grudzień'),
      'd'=>array('stycznia','lutego','marca','kwietnia',
                 'maja','czerwca','lipca','sierpnia',
                 'września','października','listopada','grudnia')
    );
    $date = $this->date;
    $date_to = $this->dateTo;
    if ($date != '') $date_str = strftime($format, $date->getTimestamp());
    else $date_str = '';
    if ($date_to != '') {
      $date_to_str = strftime($format, $date_to->getTimestamp());
      $dash = ' - ';
    } else {
      $date_to_str = '';
      $dash = '';
    }
    # changing Polish months (if exist) from denominator to genitive
    $date_str = str_replace($pl_mo['m'], $pl_mo['d'], $date_str);
    $date_to_str = str_replace($pl_mo['m'], $pl_mo['d'], $date_to_str);
    return $date_str.$dash.$date_to_str;
  }

  # get an event's time
  public function getTime($format = 'H:i') {
    return $this->formatDate($this->time, $format);
  }

  # helper function formatting date from DateTime object
  private function formatDate($data, $format) {
    if ($data != '') {
      $data = ($data instanceof DateTime) ? $data->format($format) : '';
    }
    return $data;
  }

}
